exec( "./shapeBricks.cs" );

function staticShape::onReachDestination( %this )
{
	if ( isObject( %obj = %this.spawnBrick ) )
	{
		%obj.onMoverInstructionDone();
	}
}

registerInputEvent( "fxDTSBrick", "Special", "Movers EVENT_SPECIAL_MOVERS" );
registerOutputEvent( "EVENT_SPECIAL_MOVERS", "BeginInstructions", "" );

registerOutputEvent( "fxDTSBrick", "moverStart" );
registerOutputEvent( "fxDTSBrick", "moverStop" );

function event_special_movers_beginInstructions( %obj, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 )
{
	%obj.clearMoverValues();
}

function fxDTSBrick::clearMoverValues( %this )
{
	for ( %i = 0 ; %i < %this.moverInstructionCount ; %i++ )
	{
		if ( isFunction( %func = "remove_mover_instruction_" @ %this.moverInstructionType[ %i ] ) )
		{
			call( %func, %this, %i );
		}
		
		%this.moverInstructionType[ %i ] = "";
	}
	
	%this.moverInstructionCount = "";
}

function fxDTSBrick::addMoverInstruction( %this, %type, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 )
{
	if ( !strLen( %i = %this.moverInstructionCount ) || %i < 0 )
	{
		%i = %this.moverInstructionCount = 0;
	}
	
	if ( isFunction( %func = "add_mover_instruction_" @ %type ) )
	{
		call( %func, %this, %i, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 );
	}
	
	%this.moverInstructionType[ %i ] = %type;
	%this.moverInstructionCount++;
	return true;
}

function fxDTSBrick::moverStart( %this )
{
	if ( %this.isMoverActive )
	{
		return;
	}
	
	if ( !%this.moverInstructionCount )
	{
		return;
	}
	
	if ( isObject( %this.shapeBrick ) )
	{
		%this.shapeBrick.delete();
	}
	
	if ( !isObject( %this.shapeBrick ) )
	{
		%this.shapeBrick = shapeBrick( %this );
	}
	
	%this.setColliding( false );
	%this.setRendering( false );
	%this.setRayCasting( false );
	
	%this.isMoverActive = true;
	%this.currMoverInstruction = -1;
	%this.nextMoverInstruction();
}

function fxDTSBrick::moverStop( %this )
{
	if ( !%this.isMoverActive )
	{
		return -1;
	}
	
	%this.setColliding( true );
	%this.setRendering( true );
	%this.setRayCasting( true );
	
	if ( !isObject( %shape = %this.shapeBrick ) )
	{
		return;
	}
	
	%name = %this.getName();
	%this.setName( "tempMoverBrickCopyFromName" );
	
	%obj = new fxDTSBrick( tempMoverBrickCopyToName : tempMoverBrickCopyFromName )
	{
		position = %shape.getPosition();
		rotation = %shape.getRotation();
		shapeObject = "";
		isMoverActive = "";
		performingMoverInstruction = "";
	};
	
	%obj.setName( %name );
	%this.setName( "" );
	
	%obj.setTransform( %shape.getTransform() );
	
	if ( isObject( %group = %this.getGroup() ) )
	{
		%group.add( %obj );
	}
	
	%obj.isPlanted = true;
	%obj.plant();
	
	%this.schedule( 16, "delete" );
	%shape.delete();
	
	return %obj;
}

function fxDTSBrick::onMoverInstructionDone( %this )
{
	if ( %this.isMoverActive && %this.performingMoverInstruction )
	{
		%this.performingMoverInstruction = false;
		%this.nextMoverInstruction();
	}
}

function fxDTSBrick::nextMoverInstruction( %this )
{
	if ( !%this.isMoverActive || %this.performingMoverInstruction )
	{
		return;
	}
	
	%i = %this.currMoverInstruction++;
	
	if ( %i < 0 || %i >= %this.moverInstructionCount )
	{
		%this.moverStop();
		return;
	}
	
	if ( !%this.executeMoverInstruction( %i ) )
	{
		announce( "it didn't return true, so bye, doing it again" );
		%this.nextMoverInstruction();
	}
}

function fxDTSBrick::executeMoverInstruction( %this, %index )
{
	if ( !%this.isMoverActive || %this.performingMoverInstruction )
	{
		return false;
	}
	
	if ( !isObject( %this.shapeBrick ) )
	{
		return false;
	}
	
	if ( %index < 0 || %index >= %this.moverInstructionCount )
	{
		return false;
	}
	
	%type = %this.moverInstructionType[ %index ];
	
	for ( %i = 0 ; %i < $MoverInstructionCount ; %i++ )
	{
		if ( $MoverInstructionName[ %i ] $= %type )
		{
			%found = true;
			
			if ( isFunction( %func = "start_mover_instruction_" @ %type ) )
			{
				call( %func, %this, %index );
			}
			else
			{
				%this.schedule( 16, "onMoverInstructionDone" );
			}
			
			break;
		}
	}
	
	if ( !%found )
	{
		return false;
	}
	
	%this.performingMoverInstruction = true;
	return true;
}

function registerMoverInstruction( %name, %args )
{
	if ( isMoverInstruction( %name ) )
	{
		return false;
	}
	
	if ( !strLen( %idx = $MoverInstructionCount ) || %idx < 0 )
	{
		%idx = $MoverInstructionCount = 0;
	}
	
	$MoverInstructionName[ %idx ] = %name;
	$MoverInstructionArgs[ %idx ] = %args;
	$MoverInstructionCount++;
	
	registerOutputEvent( "EVENT_SPECIAL_MOVERS", %name, %args );
	return true;
}

function isMoverInstruction( %name )
{
	for ( %i = 0 ; %i < $MoverInstructionCount ; %i++ )
	{
		if ( $MoverInstructionName[ %i ] $= %name )
		{
			return true;
		}
	}
	
	return false;
}

exec( "./instructions.cs" );

package movers_package
{
	function fxDTSBrick::onRemove( %this )
	{
		if ( isObject( %obj = %this.shapeBrick ) && !%this.removedFromShape )
		{
			%obj.removedFromBrick = true;
			%obj.delete();
		}
		
		parent::onRemove( %this );
	}
	
	function serverCmdAddEvent( %cl, %enabled, %inputEventIdx, %delay, %targetIdx, %namedTargetNameIdx, %outputEventIdx, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 )
	{
		parent::serverCmdAddEvent( %cl, %enabled, %inputEventIdx, %delay, %targetIdx, %namedTargetNameIdx, %outputEventIdx, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 );
		
		if ( !isObject( %obj = %cl.wrenchBrick ) )
		{
			return;
		}
		
		%target = getField( $InputEvent_TargetList[ "fxDTSBrick", %inputEventIdx ], %targetIdx );
		
		%input_name = $InputEvent_Name[ "fxDTSBrick", %inputEventIdx ];
		%target_name = getWord( %target, 0 );
		%output_name = $OutputEvent_Name[ getWord( %target, 1 ), %outputEventIdx ];
		
		if ( %input_name !$= "Special" )
		{
			return;
		}
		
		if ( !isFunction( %func = getWord( %target, 1 ) @ "_" @ %output_name ) )
		{
			return;
		}
		
		call( %func, %obj, %arg0, %arg1, %arg2, %arg3, %arg4, %arg5 );
	}
};

activatePackage( "movers_package" );