package shapeBrickPackage
{
	function hammerImage::onHitObject( %this, %obj, %slot, %col, %pos, %loc )
	{
		parent::onHitObject( %this, %obj, %slot, %col, %pos, %loc );
		
		if ( %col.getClassName() $= "staticShape" )
		{
			%col.killBrick();
		}
	}
	
	function paintProjectile::onCollision( %this, %obj, %col, %fade, %pos, %normal )
	{
		parent::onCollision( %this, %obj, %col, %fade, %pos, %normal );
		
		if ( %col.getClassName() $= "staticShape" )
		{
			%col.setColor( %this.colorID );
		}
	}
	
	function staticShape::onRemove( %this )
	{
		if ( isObject( %obj = %this.spawnBrick ) && !%this.removedFromBrick )
		{
			%obj.removedFromShape = true;
			%obj.delete();
		}
		
		parent::onRemove( %this );
	}
};

activatePackage( "shapeBrickPackage" );

function serverCmdBecomeShapeBrick( %cl )
{
	if ( !isObject( %cl ) || !isObject( %obj = %cl.wrenchBrick ) )
	{
		return;
	}
	
	if ( isObject( %mv = %cl.shapeBrick ) )
	{
		%mv.killBrick();
	}
	
	if ( !isObject( %mv = %cl.shapeBrick = shapeBrick( %obj ) ) )
	{
		return;
	}
	
	if ( isObject( %group = %cl.brickGroup ) )
	{
		%group.add( %mv );
	}
	
	%obj.delete();
	messageClient( %cl, '', "\c3ShapeBrick \c6successfully created." );
}

datablock staticShapeData( shapeBrickCollidingData )
{
	shapeFile = "./brick_colliding.dts";
};

datablock staticShapeData( shapeBrickNonCollidingData )
{
	shapeFile = "./brick.dts";
};

function shapeBrick( %source )
{
	if ( !isObject( %source ) )
	{
		return -1;
	}
	
	%obj = new staticShape()
	{
		position = "0 0 0";
		dataBlock = shapeBrickCollidingData;
		spawnBrick = %source;
		realColor = "0 0 0 0";
		isRendering = true;
		isRayCasting = true;
		
		moveSpeed = 1;
	};
	
	missionCleanup.add( %obj );
	%db = %source.getDataBlock();
	
	%obj.setBrickShape( %db.brickSizeX, %db.brickSizeY, %db.brickSizeZ );
	%obj.setTransform( %source.getTransform() );
	%obj.setColor( %source.colorID );
	
	%obj.setColliding( %source.isColliding() );
	%obj.setRendering( %source.isRendering() );
	%obj.setRayCasting( %source.isRayCasting() );
	
	return %obj;
}

function staticShape::stop( %this )
{
	%this.moveTarget = "";
	cancel( %this.moveTick );
}

function staticShape::moveToPosition( %this, %pos )
{
	%this.moveTarget = %pos;
	
	if ( !isEventPending( %this.moveTick ) )
	{
		%this.moveTick = %this.schedule( 16, "moveTick" );
	}
}

function staticShape::moveTick( %this )
{
	cancel( %this.moveTick );
	
	%pos = %this.getPosition();
	
	if ( !strLen( %this.moveTarget ) || !strLen( %this.moveSpeed ) || %this.moveSpeed <= 0 )
	{
		return;
	}
	
	if ( vectorDist( %pos, %this.moveTarget ) > %this.moveSpeed )
	{
		%path = vectorNormalize( vectorSub( %this.moveTarget, %pos ) );
		%itrp = vectorAdd( %pos, vectorScale( %path, %this.moveSpeed ) );
		%this.setPosition( %itrp );
	}
	else
	{
		%this.setPosition( %this.moveTarget );
		%this.moveTarget = "";
		
		%this.onReachDestination();
	}
	
	%this.moveTick = %this.schedule( 16, "moveTick" );
}

function staticShape::setMoveSpeed( %this, %speed )
{
	%this.moveSpeed = %speed / 62.5;
	
	if ( !isEventPending( %this.moveTick ) )
	{
		%this.moveTick = %this.schedule( 16, "moveTick" );
	}
}

function staticShape::killBrick( %this )
{
	%this.delete();
}

function staticShape::setBrickShape( %this, %x, %y, %z )
{
	%this.setScale( %x SPC %y SPC %z / 3 );
}

function staticShape::setColor( %this, %colorID )
{
	%this.setColorRGBA( getColorIDTable( %colorID ) );
}

function staticShape::setColorRGBA( %this, %rgba )
{
	%this.realColor = %rgba;
	
	if ( %this.isRendering )
	{
		%this.setNodeColor( "ALL", %rgba );
	}
	else
	{
		%this.setNodeColor( "ALL", "0 0 0 0" );
	}
}

function staticShape::setColliding( %this, %bool )
{
	%this.setDataBlock( %bool ? shapeBrickCollidingData : shapeBrickNonCollidingData );
	%this.setColorRGBA( %this.realColor );
}

function staticShape::setRendering( %this, %bool )
{
	%this.isRendering = %bool;
	%this.setColorRGBA( %this.realColor );
}

function staticShape::setRayCasting( %this, %bool )
{
	%this.isRayCasting = %bool;
}

function staticShape::isColliding( %this )
{
	return ( %this.getDataBlock() $= "shapeBrickNonCollidingData" ? false : true );
}

function staticShape::isRendering( %this )
{
	return ( %this.isRendering );
}

function staticShape::isRayCasting( %this )
{
	return ( %this.isRayCasting );
}

function staticShape::setPosition( %this, %pos )
{
	%this.setTransform( %pos SPC %this.getRotation() );
}

function staticShape::setRotation( %this, %rot )
{
	%this.setTransform( %this.getPosition() SPC eulerToAxis( %rot ) );
}

function staticShape::getPosition( %this )
{
	return getWords( %this.getTransform(), 0, 2 );
}

function staticShape::getRotation( %this )
{
	return axisToEuler( getWords( %this.getTransform(), 3, 6 ) );
}

function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

if ( !isFunction( "staticShape", "onReachDestination" ) )
{
	exec( "./onReachDestination.cs" );
}